from ftw.upgrade import UpgradeStep


class UpdateNewsAddableTypes(UpgradeStep):
    """Update News FTI
    """

    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.contentpage.upgrades:1009')



from ftw.contentpage.content.textblock import TextBlock
from ftw.upgrade import ProgressLogger
from ftw.upgrade import UpgradeStep
from zope.interface import alsoProvides
from zope.interface import noLongerProvides
from ftw.contentpage.interfaces import ITextBlock
from ftw.contentpage.interfaces import IContentPage
from Acquisition import aq_parent


class MigrateParagraphs(UpgradeStep):
    """Migrates simplelayout.types.common Paragraphs to
    ftw.contentpage TextBlocks.
    """

    def __call__(self):

        self.install_upgrade_profile()
        self.migrate_paragraphs()

    def install_upgrade_profile(self):
        self.setup_install_profile(
            'profile-ftw.contentpage.upgrades:1001')

    def migrate_paragraphs(self):

        try:
            from simplelayout.types.common.interfaces import IParagraph
        except ImportError:
            raise

        result = self.catalog_unrestricted_search(
            {'portal_type': 'Paragraph'}, full_objects=True)
        result = list(result)
        with ProgressLogger('Migrate Paragraphs', result) as step:
            for obj in result:
                if not IContentPage.providedBy(aq_parent(obj)):
                    continue
                self.migrate_class(obj, TextBlock)
                obj.portal_type = 'TextBlock'

                noLongerProvides(obj, IParagraph)
                alsoProvides(obj, ITextBlock)

                obj.reindexObject()
                step()



from ftw.upgrade import UpgradeStep


class DndUpload(UpgradeStep):
    """Enable dnd upload on listingblock
    """

    def __call__(self):
        self.setup_install_profile(
            'profile-collective.quickupload:default')



from ftw.contentpage.browser.baselisting import BaseListing
from ftw.contentpage.interfaces import IEventListingView
from Products.ATContentTypes.utils import DT2dt
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import implements


def format_date(start, end, wholeday):
    start = DT2dt(start)
    start_date = start.date().strftime('%d.%m.%Y')
    start_time = start.time().strftime('%H:%M')
    end = DT2dt(end)
    end_date = end.date().strftime('%d.%m.%Y')
    end_time = end.time().strftime('%H:%M')
    if wholeday:
        if start_date == end_date:
            return start_date
        return '%s - %s' % (start_date, end_date)
    if start_date == end_date:
        if start_time == end_time:
            return '%s %s' % (start_date, start_time)
        return '%s %s - %s' % (start_date, start_time, end_time)
    return '%s %s - %s %s' % (
          start_date, start_time, end_date, end_time)


class EventListing(BaseListing):
    implements(IEventListingView)

    template = ViewPageTemplateFile("eventlisting.pt")

    def get_items(self):
        # When this import is moved to the top the tests will fail
        # because of a "now()" patching problem in the test..
        from DateTime import DateTime
        query = {'path': '/'.join(self.context.getPhysicalPath()),
                 'portal_type': 'EventPage',
                 'sort_on': 'start',
                 }

        if not self.request.get('archive'):
            query['end'] = {'query': DateTime(), 'range': 'min'}
        return self.search_results(query, 'start')

    def get_date(self, brain):
        obj = brain.getObject()
        return format_date(obj.start(), obj.end(), obj.getWholeDay())



from ftw.upgrade import UpgradeStep
from zope.dottedname.resolve import resolve
from zope.interface import noLongerProvides
from ftw.upgrade.step import LOG


INTERFACE_DOTTENAME = 'ftw.contentpage.interfaces.IOrgUnitMarker'


class FixMarkerInterfaces(UpgradeStep):

    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.contentpage.upgrades:1600', steps=['rolemap', ])

        try:
            iface = resolve(INTERFACE_DOTTENAME)
            self.migrate_interface(iface)
        except ImportError:
            LOG.warn('The interface {0} does no longer exists'.format(
                INTERFACE_DOTTENAME))

    def migrate_interface(self, iface):
        query = {'object_provides': INTERFACE_DOTTENAME}
        msg = 'Migrate IOrgUnitMarker to IAuthority'

        for obj in self.objects(query, msg):
            noLongerProvides(obj, iface)
            obj.Schema()['mark_as_authority'].set(obj, True)
            obj.reindexObject(idxs=['object_provides'])



from ftw.contentpage.interfaces import IAddressBlock
from ftw.upgrade import UpgradeStep


class RemoveEmptyParagraphTagsInAddressBlocks(UpgradeStep):

    def __call__(self):
        query = {'object_provides': IAddressBlock.__identifier__}
        for obj in self.catalog_unrestricted_search(query, full_objects=True):
            if obj.getRawDirections() == '<p></p>':
                obj.setDirections('')
            if obj.getRawOpeningHours() == '<p></p>':
                obj.setOpeningHours('')



from ftw.contentpage.browser.textblock_view import TextBlockView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

class FAQBlockView(TextBlockView):

    template = ViewPageTemplateFile('faq.pt')

    def __call__(self):
        self.request['viewname'] = 'faq'
        self.portal_url = getToolByName(self.context, "portal_url")()
        return self.template()



import logging
from Products.Archetypes import atapi
from Products.CMFCore import utils
from ftw.contentpage import config
from zope.i18nmessageid import MessageFactory


_ = MessageFactory('ftw.contentpage')

logger = logging.getLogger('ftw.contentpage')

# The following patch is a workaround for a bug in Products.TinyMCE < 1.4.0:
# https://github.com/plone/Products.TinyMCE/commit/ccbe5ec5744680f73e4f64fa5d5bd00d37d51a80#comments
logger.info('Patching TinyMCE anchor search pattern.')
from Products.TinyMCE.browser import atanchors
atanchors.SEARCHPATTERN = './/a'


def initialize(context):
    """Initializer called when used as a Zope 2 product.

    This is referenced from configure.zcml. Regstrations as a "Zope 2 product"
    is necessary for GenericSetup profiles to work, for example.

    Here, we call the Archetypes machinery to register our content types
    with Zope and the CMF.
    """

    content_types, constructors, _ftis = atapi.process_types(
        atapi.listTypes(config.PROJECTNAME),
        config.PROJECTNAME)

    for atype, constructor in zip(content_types, constructors):
        utils.ContentInit('%s: %s' % (config.PROJECTNAME, atype.portal_type),
                          content_types=(atype,),
                          permission=config.ADD_PERMISSIONS[atype.portal_type],
                          extra_constructors=(constructor,),
                          ).initialize(context)



from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter
from simplelayout.base.interfaces import IBlockConfig
from zope.component import getUtility
from zope.component import queryMultiAdapter
from simplelayout.base.interfaces import IScaleImage


class TextBlockView(BrowserView):

    def __init__(self, context, request):
        super(TextBlockView, self).__init__(context, request)
        self.portal_state = getMultiAdapter((self.context, self.request),
                                            name=u'plone_portal_state')

        self.blockconf = IBlockConfig(self.context)
        self.image_layout = self.blockconf.image_layout

    def get_css_klass(self):
        if 'image' not in self.context.Schema():
            return ''

        layout = self.image_layout
        return 'sl-img-' + layout

    def has_image(self):
        if 'image' not in self.context.Schema():
            return False
        return bool(self.context.getImage())

    def has_teaser(self):
        """Returns True when the current context uses the teaser_schema.
        """
        return 'teaserSelectLink' in self.context.Schema()

    def get_image_tag(self):
        alt = self.context.getImageAltText()
        title = unicode(self.context.getImageCaption(),
                        self.context.getCharset())

        if not alt:
            # Note: The file name might be a unicode string.
            alt = self.context.getImage().filename or ''

        if not isinstance(alt, unicode):
            alt = unicode(alt, self.context.getCharset())

        image_util = getUtility(
            IScaleImage,
            name='simplelayout.image.scaler')
        img_attrs = image_util.get_image_attributes(self.context)
        scales = queryMultiAdapter((self.context, self.request), name="images")

        if img_attrs['width'] == 0 and img_attrs['height'] == 0:
            # either there is no image or we use a layout such as
            # "no-image" which does not show the image - we do not
            # need to create a scale in this case nor to return a
            # <img> HTML tag.
            return ''

        return scales.scale(
            'image',
            width=img_attrs['width'],
            height=img_attrs['height']).tag(title=title, alt=alt)

    def image_wrapper_style(self):
        """ sets width of the div wrapping the image, so the
        caption will shown under the image
        """

        image_util = getUtility(
            IScaleImage,
            name='simplelayout.image.scaler')
        img_attrs = image_util.get_image_attributes(self.context)
        return "width: %spx" % img_attrs['width']

    def get_block_height(self):
        height = self.blockconf.block_height
        return height and 'height: %spx' % height or ''

    def get_image_url(self):
        if not self.has_teaser():
            return None

        teaser_url = self.get_teaser_url()
        if teaser_url:
            return teaser_url

        if self.context.getImageClickable():
            return self.context.absolute_url() + '/image'

        return None

    def get_teaser_url(self):
        if not self.has_teaser():
            return None

        linkSelector = self.context.getTeaserSelectLink()

        if linkSelector == 'intern':
            ref_obj = self.context.getTeaserReference()

            return ref_obj and ref_obj.absolute_url() or None

        if linkSelector == 'extern':
            url = self.context.getTeaserExternalUrl()

            return url != '' and url or None

        return None



from ftw.builder import Builder
from ftw.builder import create
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from unittest2 import TestCase


class TestAnchors(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    @browsing
    def test_anchors(self, browser):
        """
        This test makes sure that `ContentPageAnchorView` is able
        to extract the anchors form the text blocks of a content page.
        """
        contentpage = create(Builder('content page').titled('The Page'))
        create(Builder('text block')
               .within(contentpage)
               .having(text="<p>Lorem <a name='anchor-textblock1'></a> "
                            "ipsum dolor sit amet.</p>"))
        create(Builder('text block')
               .within(contentpage)
               .having(text="<p>Lorem <a name='anchor-textblock2'></a> "
                            "ipsum dolor sit amet.</p>"))

        view = contentpage.restrictedTraverse('content_anchors')
        anchor_names = view.listAnchorNames()
        self.assertEqual(['anchor-textblock1', 'anchor-textblock2'],
                         anchor_names)



from collective.geo.settings.interfaces import IGeoCustomFeatureStyle
from zope.component import queryAdapter


def initializeCustomFeatureStyles(obj, event):
    """Initializes IGeoCustomFeatureStyle for AddressBlocks
    upon object creation.

    For AddressBlocks we don't want to display the map viewlet in one of the
    default viewlet managers but only in the custom 'contact_view'.
    """
    custom_styles = queryAdapter(obj, IGeoCustomFeatureStyle)
    custom_styles.set('use_custom_styles', True)
    custom_styles.set('map_height', '30em')
    custom_styles.set('map_viewlet_position', 'fake-manager')  # Don't display



from Acquisition import aq_inner
from Acquisition import aq_parent
from collective.geo.geographer.interfaces import IGeoreferenced
from collective.geo.mapwidget.browser.widget import MapWidget
from ftw.contentpage.interfaces import IAddressBlockView
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import implements


class AddressBlockView(BrowserView):
    """Block representation of AddressBlock"""
    implements(IAddressBlockView)

    address_ptl = ViewPageTemplateFile('address.pt')

    def get_address_as_html(self):
        html = [self.context.getAddress()]
        if self.context.getExtraAddressLine():
            html.append(self.context.getExtraAddressLine())
        return '<br />'.join(html)

    def address(self):
        return self.address_ptl()

    def get_address_map(self):
        address_map = MapWidget(self, self.request, self.context)
        address_map.mapid = "geo-%s" % self.context.getId()
        address_map.addClass('addressblock-map')

        return address_map

    def show_map(self):
        return IGeoreferenced(self.context).hasCoordinates()


class AddressBlockPortletView(AddressBlockView):
    """Block representation of AddressBlock"""

    def has_team(self):
        result = aq_parent(aq_inner(self.context)).getFolderContents(
            contentFilter={'portal_type': 'ContentPage',
                           'id': 'team'})
        return bool(result)


class AddressBlockDetailView(AddressBlockView):
    """AddressBlock detail View"""



from DateTime import DateTime
from ftw.contentpage.browser.eventlisting import format_date
import unittest2 as unittest


class TestEvent(unittest.TestCase):

    def test_same_date(self):
        start = DateTime(2013, 01, 01, 18, 00)
        end = DateTime(2013, 01, 01, 19, 00)
        wholeday = False

        self.assertEquals(format_date(start, end, wholeday),
            '01.01.2013 18:00 - 19:00')

    def test_diff_date(self):
        start = DateTime(2013, 01, 01, 18, 00)
        end = DateTime(2013, 01, 03, 19, 00)
        wholeday = False

        self.assertEquals(format_date(start, end, wholeday),
            '01.01.2013 18:00 - 03.01.2013 19:00')

    def test_same_date_wholeday(self):
        start = DateTime(2013, 01, 01)
        end = DateTime(2013, 01, 01)
        wholeday = True

        self.assertEquals(format_date(start, end, wholeday),
            '01.01.2013')

    def test_diff_date_wholeday(self):
        start = DateTime(2013, 01, 01)
        end = DateTime(2013, 01, 03)
        wholeday = True

        self.assertEquals(format_date(start, end, wholeday),
            '01.01.2013 - 03.01.2013')

    def test_same_data_same_time(self):
        start = DateTime(2013, 01, 01, 11, 00)
        end = DateTime(2013, 01, 01, 11, 00)
        wholeday = False

        self.assertEquals(format_date(start, end, wholeday),
            '01.01.2013 11:00')



from hashlib import md5
from plone.i18n.normalizer.interfaces import IIDNormalizer
from Products.CMFCore.utils import getToolByName
from zope.component import queryUtility
from zope.interface import directlyProvides
from zope.schema import vocabulary
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm


def SubjectVocabulary(context):
    """Returns a vocabulary of the available suppliers
    """
    # context is the portal config options, whose context is the portal
    catalog = getToolByName(context, 'portal_catalog')
    normalizer = queryUtility(IIDNormalizer)
    terms = []

    for term in catalog.uniqueValuesFor("Subject"):
        # Just the normalized term is not enought for an unique token.
        # The unique values are case sensitive. After normalizing the term,
        # we get id's in lower-case. That means, we lose the unique value after
        # normalizing if we get values like: "Bond" and "bonD". Adding an md5
        # hash to the normalized term, we prevent double tokens and the
        # token is still readable.
        token = '%s-%s' % (md5(term).hexdigest(), normalizer.normalize(term))
        terms.append(SimpleTerm(value=term,
                                token=token,
                                title=term.decode('utf8')))

    return vocabulary.SimpleVocabulary(terms)

directlyProvides(SubjectVocabulary, IVocabularyFactory)


def contentcategories_vocabulary(context):
    normalizer = queryUtility(IIDNormalizer)
    catalog = getToolByName(context, 'portal_catalog')
    terms = []
    for term in catalog.uniqueValuesFor("getContentCategories"):
        terms.append(SimpleTerm(value=term.decode('utf8'),
                                token=normalizer.normalize(term.decode('utf8')),
                                title=term.decode('utf8')))
    return vocabulary.SimpleVocabulary(terms)

directlyProvides(contentcategories_vocabulary, IVocabularyFactory)



from ftw.upgrade import UpgradeStep


class UpdateAddressBlockFTI(UpgradeStep):
    """Update AddressBlock FTI
    """

    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.contentpage.upgrades:1007')



from datetime import datetime
from ftw.builder import create
from ftw.builder import Builder
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from pytz import timezone as tz
from unittest2 import TestCase


class TestNewsEffective(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.newsfolder = create(Builder('news folder'))

    def test_effective(self):
        self.news = create(Builder('news').within(self.newsfolder))
        difference = datetime.now(tz('Europe/Zurich')) -\
            self.news.effective().asdatetime()
        # The folllowing Calculation is the same as timedelta.total_seconds()
        # but since it isn't available in python2.6 we need to calculate it ourself
        # https://docs.python.org/2/library/datetime.html#datetime.timedelta.total_seconds
        self.assertLess((difference.microseconds +
                        (difference.seconds + difference.days * 24 * 3600)
                        * 10 ** 6) / 10.0 ** 6, 60)



from ftw.builder import Builder
from ftw.builder import create
from ftw.builder import registry
from ftw.builder.dexterity import DexterityBuilder
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from pkg_resources import get_distribution
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.fti import DexterityFTI
from Products.CMFCore.utils import getToolByName
from simplelayout.base.views import SimpleLayoutView
from unittest2 import skipUnless
from unittest2 import TestCase
from zope import schema
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.interface import Interface
from zope.viewlet.interfaces import IViewletManager


class ISampleDX(Interface):
    title = schema.TextLine(
        title=u'Title',
        required=False)

alsoProvides(ISampleDX, IFormFieldProvider)


class SampleBuilder(DexterityBuilder):
    portal_type = 'Sample'

registry.builder_registry.register('sample', SampleBuilder)


@skipUnless(get_distribution('Plone').version >= '4.3', 'requires plone 4.3')
class TestContentCategoriesBehavior(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        # add sample fti
        self.fti = DexterityFTI('Sample')
        self.fti.schema = 'ftw.contentpage.tests.test_content_categorie_behavior.ISampleDX'
        self.fti.behaviors = (
            'ftw.contentpage.behaviors.content_categories.IContentCategories', )

        self.portal.portal_types._setObject('Sample', self.fti)

    def test_category_index(self):
        catalog = getToolByName(self.portal, 'portal_catalog')

        create(Builder('sample')
               .having(content_categories=(u'DEMO1', )))

        self.assertTrue(catalog({'getContentCategories': 'DEMO1'})[0])

    def test_category_index_umlauts(self):
        catalog = getToolByName(self.portal, 'portal_catalog')

        create(Builder('sample')
               .having(content_categories=(u'WITH unicode \xe4', )))

        unique_values = catalog.Indexes['getContentCategories'].uniqueValues()
        self.assertIn("WITH unicode \xc3\xa4", unique_values)

    def _get_viewlet(self, obj):
        view = SimpleLayoutView(obj, obj.REQUEST)
        manager_name = 'simplelayout.base.listing'
        manager = queryMultiAdapter(
            (obj, obj.REQUEST, view),
            IViewletManager,
            manager_name)
        self.failUnless(manager)
        # Set up viewlets
        manager.update()
        name = 'ftw.contentpage.contentlisting'
        return [v for v in manager.viewlets if v.__name__ == name]

    def test_viewlet_with_dx_content(self):
        page = create(Builder('content page'))
        sampledxcontent = create(Builder('sample')
                                 .titled('Democontent')
                                 .within(page)
                                 .having(content_categories=(u'WITH unicode \xe4', )))

        viewlet = self._get_viewlet(page)[0]
        self.assertTrue(viewlet.available())

        self.assertIn(
            ('WITH unicode \xc3\xa4',
                [('Democontent', sampledxcontent.absolute_url(), '')]),
            viewlet.get_content())

    def test_adding_new_category_using_the_new_categories_field(self):
        page = create(Builder('content page'))
        sampledxcontent = create(Builder('sample')
                                 .titled('Democontent')
                                 .within(page)
                                 .having(new_content_categories=(u'WITH unicode \xe4', )))

        viewlet = self._get_viewlet(page)[0]
        self.assertTrue(viewlet.available())

        self.assertIn(
            ('WITH unicode \xc3\xa4',
                [('Democontent', sampledxcontent.absolute_url(), '')]),
            viewlet.get_content())

    @browsing
    def test_adding_new_categories_only_for_managers(self, browser):
        page = create(Builder('content page'))
        user = create(Builder('user')
                      .with_roles('Site Administrator', on=page))

        sampledxcontent = create(Builder('sample')
                                 .titled('Democontent')
                                 .within(page))

        browser.login(user.getId()).visit(sampledxcontent, view='@@edit')

        selector = '#formfield-form-widgets-IContentCategories-new_content_categories'
        self.assertFalse(browser.css(selector),
                         'New categories field should no be visible.')

        browser.login().visit(sampledxcontent, view='@@edit')
        self.assertTrue(browser.css(selector),
                         'New categories field should be visible.')



from Products.CMFPlone.utils import getToolByName
from Products.Five.browser import BrowserView
from ftw.contentpage import _
from ftw.table import helper
from ftw.table.interfaces import ITableGenerator
from plone.registry.interfaces import IRegistry
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.component import queryUtility, getUtility
import pkg_resources


try:
    pkg_resources.get_distribution('ftw.file')
except pkg_resources.DistributionNotFound:
    HAS_FILE = False
else:
    HAS_FILE = True


def file_link(icon=True, icon_only=False):

    def _helper(item, value):
        registry = getUtility(IRegistry)
        default_view_name = registry.get('ftw.contentpage.listingblock.defaultfileviewname', u'').encode('utf-8')

        # Build the url without a trailing slash.
        url_fragments = [item.getURL()]
        if default_view_name:
            url_fragments.append(default_view_name)
        url = '/'.join(url_fragments)

        attrs = {}
        attrs['href'] = url
        attrs['title'] = item.Description

        return helper.linked(item, value, show_icon=icon, attrs=attrs,
                             icon_only=icon_only)
    return _helper


class ListingBlockView(BrowserView):
    """Block representation of ListingBlock"""

    def __init__(self, context, request):
        super(ListingBlockView, self).__init__(context, request)
        self.sort_on = 'sortable_title'
        self.sort_order = 'asc'

    def columns(self):
        columns = (
            {'column': 'getContentType',
             'column_title': _(u'column_type', default=u'Type'),
             'sort_index': 'getContentType',
             'transform': file_link(icon=True, icon_only=True)},

            {'column': 'Title',
             'column_title': _(u'column_title', default=u'Title'),
             'sort_index': 'sortable_title',
             'transform': file_link(icon=False)},

            {'column': 'modified',
             'column_title': _(u'column_modified', default=u'modified'),
             'sort_index': 'modified',
             'transform': helper.readable_date,
             },

            {'column': 'Creator',
             'column_title': _(u'column_creater', default=u'creater'),
             'transform': helper.readable_author,
             },

            {'column': 'getObjSize',
             'column_title': _(u'column_size', default=u'size'),
             },

            {'column': 'review_state',
             'column_title': _(u'review_state', default=u'Review State'),
             'transform': helper.translated_string(),
             },

            {'column': 'id',
             'column_title': _(u'ID', default=u'ID'),
             'sort_index': 'id',
             },
            )

        if HAS_FILE:
            columns += (
                {'column': 'documentDate',
                 'column_title': _(u'column_documen_date',
                                   default=u'Document date'),
                 'sort_index': 'documentDate',
                 'transform': helper.readable_date},)

        return columns

    def block_visible(self):
        mtool = getToolByName(self.context, 'portal_membership')
        if mtool.checkPermission('Modify portal content', self.context):
            return True

        return len(self.get_table_contents()) > 0

    @property
    def _build_query(self):
        query = {}
        path = '/'.join(self.context.getPhysicalPath())
        query['path'] = {'query': path, 'depth': 1}
        query['sort_on'] = self.context.getSortOn()
        query['sort_order'] = self.context.getSortOrder()
        return query

    def _get_column(self, column):
        for col in self.columns():
            if column == col['column']:
                return col
        return None

    def _filtered_columns(self):
        filtered = []

        for col in self.context.getTableColumns():
            column = self._get_column(col)
            if column:
                filtered.append(column)

        return filtered

    def get_table_contents(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        return catalog(self._build_query)

    def render_table(self):
        # Use a custom table template, because we don't want a table header id.
        # The id value is moved to a css klass.
        # Reason: It's no allowed to have an id more than once (In case we
        # have more than one Listingblock on one contentpage)
        template = ViewPageTemplateFile('table-custom-template.pt')

        generator = queryUtility(ITableGenerator, 'ftw.tablegenerator')
        return generator.generate(self.get_table_contents(),
                                  self._filtered_columns(),
                                  sortable=True,
                                  template=template,
                                  options={'table_caption': self.context.title},
                                  selected=(self._build_query['sort_on'],
                                            self._build_query['sort_order']))



from ftw.builder import Builder
from ftw.builder import create
from ftw.contentpage.interfaces import IAuthority
from ftw.contentpage.testing import FTW_CONTENTPAGE_INTEGRATION_TESTING
from Products.CMFCore.utils import getToolByName
from unittest2 import TestCase


class TestMarkerInterfaceForListings(TestCase):

    layer = FTW_CONTENTPAGE_INTEGRATION_TESTING

    def setUp(self):
        super(TestMarkerInterfaceForListings, self).setUp()

        self.portal = self.layer['portal']

    def test_contentpage_is_extended_with_checkbox(self):
        page = create(Builder('content page'))

        fieldname = 'mark_as_authority'
        self.assertIn(fieldname,
                      page.Schema().keys(),
                      '{0} nod found in content page schema'.format(fieldname))

    def test_authority_marker_is_set_if_checkbox_is_true(self):
        page = create(Builder('content page'))
        page.Schema()['mark_as_authority'].set(page, True)

        self.assertTrue(IAuthority.providedBy(page),
                        '{0} is not marked for listings'.format(page))

    def test_authority_marker_is_not_set_if_checkbox_is_false(self):
        page = create(Builder('content page'))
        page.Schema()['mark_as_authority'].set(page, False)

        self.assertFalse(IAuthority.providedBy(page),
                         '{0} is marked for listings'.format(page))

    def test_listing_marker_is_not_set_by_default(self):
        page = create(Builder('content page'))

        self.assertFalse(IAuthority.providedBy(page),
                         '{0} is marked for listings'.format(page))

    def test_catalog_is_always_up_to_date(self):
        catalog = getToolByName(self.portal, 'portal_catalog')
        page = create(Builder('content page'))

        result = catalog({'object_provides': IAuthority.__identifier__})
        self.assertFalse(result, 'Nothing should be marked by default.')

        page.Schema()['mark_as_authority'].set(page, True)
        page.processForm()

        result = catalog({'object_provides': IAuthority.__identifier__})
        self.assertTrue(result, 'Content should be marked')

        page.Schema()['mark_as_authority'].set(page, False)
        page.processForm()

        result = catalog({'object_provides': IAuthority.__identifier__})
        self.assertFalse(result, 'Nothing should be marked by default.')

    def test_mark_for_listing_permission_default(self):
        permission = 'ftw.contentpage: Toggle IAuthority marker interface'

        roles = [r['name'] for r in self.portal.rolesOfPermission(permission) if r[
            'selected']]
        self.assertEquals(roles,
                          ['Contributor', 'Manager'],
                          'Manager and Contributor should have the permission:'
                           ' {0}'.format(
                              permission))

    def test_mark_for_listing_permission(self):
        page = create(Builder('content page'))

        permission = 'ftw.contentpage: Toggle IAuthority marker interface'
        page.manage_permission(permission, roles=[], acquire=False)

        roles = [r['name'] for r in page.rolesOfPermission(permission) if r[
            'selected']]
        self.assertEquals(roles,
                          [],
                          'No one should have the permission: {0}'.format(
                              permission))



from Acquisition import IAcquirer, aq_parent, aq_inner
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.contentpage import _
from ftw.contentpage.browser.baselisting import BaseListing
from ftw.contentpage.interfaces import INewsListingView
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletRenderer
from plone.portlets.interfaces import IPortletManager
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.interface import implements


class NewsListing(BaseListing):
    implements(INewsListingView)

    default_template = ViewPageTemplateFile('newslisting.pt')
    rss_template = ViewPageTemplateFile('newslisting_rss.pt')

    @property
    def template(self):
        if self.__name__ == 'news_rss_listing':
            return self.rss_template
        else:
            return self.default_template

    def show_author(self):
        """Checks if the user is anonymous and is not allowAnonymousViewAbout.
        """
        site_props = getToolByName(self.context,
                                   'portal_properties').site_properties
        mt = getToolByName(self.context, 'portal_membership')

        if not site_props.getProperty('allowAnonymousViewAbout', False) \
                and mt.isAnonymousUser():
            return False
        return True

    def get_items(self):
        """Get all news items"""
        query = {}
        query['object_provides'] = 'ftw.contentpage.interfaces.INews'
        query['sort_on'] = 'effective'
        query['sort_order'] = 'reverse'

        # Implement archive functionality - used by the archive portlet
        return self.search_results(query, 'effective')

    def title(self):
        title = self.context.Title()
        if self.context.portal_type == 'NewsFolder':
            return title
        else:
            return self.context.Title() + ' - News'

    def link(self):
        return self.context.absolute_url() + '/' + self.__name__

    def description(self):
        return _(u'label_feed_desc',
                 default=u'${title} - News Feed',
                 mapping={'title': self.context.Title().decode('utf-8')})


class NewsPortletListing(NewsListing):

    def get_portlet(self):
        manager_name = self.request.form.get('manager', None)
        name = self.request.form.get('portlet', None)
        if not manager_name or not name:
            return

        managers_and_assignments = self.get_manager_and_assignments(
            manager_name
        )
        for manager, assignments in managers_and_assignments:
            if name in assignments:
                return queryMultiAdapter(
                    (self.context, self.request, self,
                     manager, assignments[name]),
                    IPortletRenderer)
        return

    def get_manager_and_assignments(self, manager_name):
        context = self.context

        # Prepare a list of objects by walking up the path.
        contexts = [context]
        while not IPloneSiteRoot.providedBy(context):
            context = aq_parent(aq_inner(context))
            contexts.append(context)

        # Prepare a list of tuples in the form `(manager, assignments)`.
        managers_and_assignments = []
        for context in contexts:
            manager = getUtility(
                IPortletManager,
                name=manager_name,
                context=context)
            assignments = getMultiAdapter(
                (context, manager),
                IPortletAssignmentMapping,
                context=context)

            if assignments is not None:
                managers_and_assignments.append((manager, assignments))

        return managers_and_assignments

    def get_items(self):
        portlet = self.get_portlet()
        if portlet:
            # Get the news items from the portlet. Tell the portlet that
            # it is the listing view who asks for the news items.
            return portlet.get_news(all_news=True, listing_view=True)

        return []

    def title(self):
        portlet = self.get_portlet()
        if portlet:
            return portlet.data.portlet_title
        return super(NewsPortletListing, self).title()



from ftw.contentpage.testing import FTW_CONTENTPAGE_INTEGRATION_TESTING
from unittest2 import TestCase
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility
from plone.registry.interfaces import IRegistry
from collective.geo.settings.interfaces import IGeoSettings
from collective.geo.settings.interfaces import IGeoFeatureStyle
from plone.portlets.constants import CONTENT_TYPE_CATEGORY
from plone.portlets.interfaces import IPortletManager


class TestAddressGeoConfigHandler(TestCase):

    layer = FTW_CONTENTPAGE_INTEGRATION_TESTING

    def setUp(self):
        super(TestAddressGeoConfigHandler, self).setUp()
        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()

    def test_setuphandler_called_again(self):
        setup = getToolByName(self.portal, 'portal_setup')
        setup.runAllImportStepsFromProfile(
            'profile-ftw.contentpage:default',
            ignore_dependencies=True,
            purge_old=False)

        # Check geo settings
        registry = getUtility(IRegistry)
        geo_content_types = registry.forInterface(
            IGeoSettings).geo_content_types
        self.assertIn('AddressBlock', geo_content_types)

        map_viewlet_position = registry.forInterface(
            IGeoFeatureStyle).map_viewlet_position
        self.assertEquals(map_viewlet_position, 'fake-manager')

        # Check dropzone portlet - There should only be one
        manager = getUtility(IPortletManager, name=u"plone.rightcolumn")
        mapping = manager[CONTENT_TYPE_CATEGORY][u"ContentPage"]
        self.assertEquals(len(mapping), 1)



from ftw.contentpage.portlets import news_portlet
from ftw.upgrade import UpgradeStep
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletManager
from zope.component import getMultiAdapter
from zope.component import getUtility


class UpgradeNewsPortlet(UpgradeStep):
    """Upgrade news portlets: add always_render_portlet default value to assignments.

    When news portlets were added to the Plone site before ftw.contentpage 1.9.0,
    the assignment has no always_render_portlet attribute.
    When exporting portlets with Generic Setup, the Plone site assignments are
    exported. If the attribute is missing, an AttributeError is raised.
    """

    def __call__(self):
        for assignment in self.news_portlet_assignments_for(self.portal):
            assignment.always_render_portlet = getattr(
                assignment, 'always_render_portlet', False)

    def news_portlet_assignments_for(self, context):
        for assignment in self.get_assignments_for(self.portal):
            if isinstance(assignment, news_portlet.Assignment):
                yield assignment

    def get_assignments_for(self, context):
        for manager_name in ('plone.leftcolumn', 'plone.rightcolumn'):
            manager = getUtility(IPortletManager, name=manager_name,
                                 context=self.portal)
            for assignment in getMultiAdapter((context, manager),
                                              IPortletAssignmentMapping,
                                              context=self.portal).values():
                yield assignment



#from DateTime import DateTime
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from plone.testing.z2 import Browser
import transaction
import unittest2 as unittest


class TestNewsViewlet(unittest.TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.newsfolder = self.portal.get(
            self.portal.invokeFactory('NewsFolder', 'Newsfolder'))
        self.news = self.newsfolder.get(
            self.newsfolder.invokeFactory('News', 'News1'))
        transaction.commit()
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

    def _auth(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

    def test_event_date_single_day(self):
        self._auth()
        self.browser.open(self.news.absolute_url())
        self.assertIn('<div class="newsPublicationDate">', self.browser.contents)



from StringIO import StringIO
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from plone.app.imaging.utils import getAllowedSizes
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD
from plone.app.testing import login
from plone.app.testing import logout
from plone.testing.z2 import Browser
from unittest2 import TestCase
from zope.component import queryMultiAdapter
import os
import transaction


class TestListingBlockViews(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestListingBlockViews, self).setUp()
        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()

        self.contentpage = self.portal.get(
            self.portal.invokeFactory('ContentPage', 'contentpage'))
        # Fire all necessary events
        self.contentpage.processForm()
        transaction.commit()

        # Browser setup
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

    def _auth(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

    def _create_listingblock(self):
        listingblock = self.contentpage.get(
            self.contentpage.invokeFactory('ListingBlock', 'listingblock'))
        # Fire all necessary events
        listingblock.processForm()
        transaction.commit()
        return listingblock

    def get_columns(self, columns):
        return [col['column'] for col in columns]

    def test_listing_view_registration(self):
        listingblock = self._create_listingblock()
        view = queryMultiAdapter((listingblock, listingblock.REQUEST),
                                 name="block_view")
        self.assertNotEquals(view, None)

    def test_gallery_view_registration(self):
        listingblock = self._create_listingblock()
        view = queryMultiAdapter((listingblock, listingblock.REQUEST),
                                 name="block_view-gallery")
        self.assertNotEquals(view, None)

    def test_listingblock_view(self):
        listingblock = self._create_listingblock()
        self._auth()
        self.browser.open(self.contentpage.absolute_url())
        self.assertIn(listingblock.getId(), self.browser.contents)
        self.assertIn('simplelayout-block-wrapper ListingBlock',
                      self.browser.contents)

    def test_empty_block_invisible_for_anonymous(self):
        listingblock = self._create_listingblock()
        logout()
        view = queryMultiAdapter((listingblock, listingblock.REQUEST),
                                 name="block_view")
        self.assertEqual(
            '', view().strip(),
            'Listing block should not be visible for anonymous when empty.')

    def test_listing_block_table_render(self):
        listingblock = self._create_listingblock()
        _file = listingblock.get(
            listingblock.invokeFactory('File', 'file'))
        dummy = StringIO("DATA")
        dummy.filename = 'dummy.pdf'
        _file.setFile(dummy)
        _file.setTitle("Dummy PDF")
        _file.setDescription('Dummy description')
        _file.processForm()

        view = queryMultiAdapter((listingblock, listingblock.REQUEST),
                                 name='block_view')
        self.assertIn("Dummy PDF", view.render_table())
        self.assertIn("pdf.png", view.render_table())
        self.assertIn("Dummy description", view.render_table())

    def test_custom_table_template(self):
        listingblock = self._create_listingblock()
        _file = listingblock.get(
            listingblock.invokeFactory('File', 'file'))
        dummy = StringIO("DATA")
        dummy.filename = 'dummy.pdf'
        _file.setFile(dummy)
        _file.setTitle("Dummy PDF")
        _file.processForm()

        view = queryMultiAdapter((listingblock, listingblock.REQUEST),
                                 name='block_view')

        # The table header (th) id value is implemented as css klass
        self.assertNotIn('id="header-getContentType"', view.render_table())
        self.assertNotIn('id="header-sortable_title"', view.render_table())
        self.assertNotIn('id="header-modified"', view.render_table())

        self.assertIn('class="header-getContentType', view.render_table())
        self.assertIn('class="header-sortable_title', view.render_table())
        self.assertIn('class="header-modified', view.render_table())

        self.assertIn('class="column-getContentType', view.render_table())
        self.assertIn('class="column-sortable_title', view.render_table())
        self.assertIn('class="column-modified', view.render_table())

    def test_listing_block_get_column(self):
        listingblock = self._create_listingblock()
        view = queryMultiAdapter((listingblock, listingblock.REQUEST),
                                 name="block_view")
        column = view._get_column('Title')
        self.assertEquals(column['column'], 'Title')
        self.assertEquals(column['sort_index'], 'sortable_title')

        # Get inexisting column
        column = view._get_column('Dummy')
        self.assertIsNone(column)

    def test_listing_block_filtered_columns(self):
        listingblock = self._create_listingblock()
        view = queryMultiAdapter((listingblock, listingblock.REQUEST),
                                 name="block_view")
        columns = view._filtered_columns()

        self.assertEquals(
            ['getContentType', 'Title', 'modified', 'Creator',
             'getObjSize', 'review_state', 'id'],
            self.get_columns(view.columns()))
        self.assertEquals(
            ['getContentType', 'Title', 'modified'],
            self.get_columns(columns))

    def test_listing_block_filtered_wrong_column_name(self):
        listingblock = self._create_listingblock()
        listingblock.setTableColumns(['Dummy'])
        view = queryMultiAdapter((listingblock, listingblock.REQUEST),
                                 name="block_view")
        columns = view._filtered_columns()
        self.assertEquals(
            ['getContentType', 'Title', 'modified', 'Creator',
             'getObjSize', 'review_state', 'id'],
            self.get_columns(view.columns()))
        self.assertEquals([], self.get_columns(columns))

    def test_scale_installed(self):
        listingblock = self._create_listingblock()
        _image = listingblock.get(
            listingblock.invokeFactory('Image', 'image'))
        scales = _image.restrictedTraverse('@@images')
        self.assertIn('listingblock_gallery', scales.getAvailableSizes())

    def test_gallery_get_box_boundaries(self):
        listingblock = self._create_listingblock()
        view = queryMultiAdapter((listingblock, listingblock.REQUEST),
                         name='block_view-gallery')
        self.assertEquals(getAllowedSizes()['listingblock_gallery'],
            view._get_box_boundaries())

    def test_gallery_get_images(self):
        listingblock = self._create_listingblock()
        _image = (listingblock.get(
            listingblock.invokeFactory('Image', 'image')))
        listingblock.invokeFactory('File', 'file')
        view = queryMultiAdapter((listingblock, listingblock.REQUEST),
                         name='block_view-gallery')
        #Only return images
        self.assertEquals([_image], view.get_images())

    def test_gallery_view(self):
        listingblock = self._create_listingblock()
        _image = listingblock.get(
            listingblock.invokeFactory('Image', 'image'))

        dummy = open("%s/dummy.png" % os.path.split(__file__)[0], 'r')
        dummy.seek(0)
        _image.setImage(dummy)
        _image.setTitle('Dummy Image')
        _image.processForm()

        view = queryMultiAdapter((listingblock, listingblock.REQUEST),
                                 name='block_view-gallery')
        self.assertIn("<img src=\"http://nohost/plone/contentpage/"
            "listingblock/image/@@images",
                      view.index())
        self.assertIn("title=\"Dummy Image\"", view.index())

    def tearDown(self):
        super(TestListingBlockViews, self).tearDown()
        portal = self.layer['portal']
        login(portal, TEST_USER_NAME)
        portal.manage_delObjects(['contentpage'])

        transaction.commit()



from zope.interface import Interface


class IContentPage(Interface):
    """Marker interface for ContenPages"""


class IAddressBlock(Interface):
    """Marker interface for AddressBlocks"""


class IListingBlock(Interface):
    """Marker interface for AddressBlocks"""


class IAddressBlockView(Interface):
    """Marker interface for AddressBlocks"""


class ITextBlock(Interface):
    """Marker interface for TextBlocks"""


class IOrgUnitMarker(Interface):
    """Marker interface for AddressBlocks"""


class IAuthoritySupport(Interface):
    """Marker interface for authorities"""


class IAuthority(Interface):
    """Marker interface for listing checkbox"""


class ICategorizable(Interface):
    """Marker interface for categorizable content"""


class INews(Interface):
    """Marker interface for news type"""


class INewsFolder(Interface):
    """Marker interface for newsfolder type"""


class IEventPage(Interface):
    """Marker interface for Event Type"""


class IEventFolder(Interface):
    """Marker interface for Event Type"""


class ITeaser(Interface):
    """Markter interface for the teaser functionality"""


class INewsListingView(Interface):
    """Marker interface for the news listing view"""


class IEventListingView(Interface):
    """Marker interface for the news listing view"""


class ISummaryListingEnabled(Interface):
    """Marker interface for the detail content listing view"""


class IFtwContentPageLayer(Interface):
    """Request marker for ftw.contentpage"""



from DateTime import DateTime
from ftw.builder import Builder
from ftw.builder import create
from ftw.contentpage.portlets.base_archive_portlet import ArchiveSummary
from ftw.contentpage.testing import FTW_CONTENTPAGE_INTEGRATION_TESTING
from plone.app.testing import setRoles, TEST_USER_ID, TEST_USER_NAME, login
from unittest2 import TestCase


class TestArchiveSummary(TestCase):

    layer = FTW_CONTENTPAGE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

        self.archive_summary = ArchiveSummary(
            self.portal,
            self.request,
            ['ftw.contentpage.interfaces.INews'],
            'effective',
            'newslisting')

    def _create_news(self, date_str):
        create(Builder('news')
               .having(effectiveDate=DateTime(date_str)))

    def test_no_news_to_list(self):
        self.assertEquals([], self.archive_summary())

    def test_months_are_sorted_from_newer_to_older(self):
        self._create_news('2013/01/01')
        self._create_news('2013/08/01')
        self._create_news('2013/03/01')

        months = self.archive_summary()[0].get('months')

        self.assertEquals(
            [u'August', u'March', u'January'],
            [month.get('title') for month in months])

    def test_years_are_sorted_from_newer_to_older(self):
        self._create_news('2011/01/01')
        self._create_news('2015/08/01')
        self._create_news('2013/03/01')

        result = self.archive_summary()

        self.assertEquals(
            ['2015', '2013', '2011'],
            [year.get('title') for year in result])

    def test_count_all_news_for_each_year_in_the_title(self):
        self._create_news('2011/01/01')
        self._create_news('2011/08/01')
        self._create_news('2011/03/01')
        self._create_news('2012/03/01')
        self._create_news('2012/08/01')

        result = self.archive_summary()

        self.assertEquals(
            [2, 3],
            [year.get('number') for year in result])

    def test_count_all_news_for_each_month(self):
        self._create_news('2013/01/01')
        self._create_news('2013/01/04')
        self._create_news('2013/01/06')
        self._create_news('2013/02/06')
        self._create_news('2013/02/04')
        self._create_news('2012/02/04')

        result = self.archive_summary()

        months_2013 = result[0].get('months')
        months_2012 = result[1].get('months')

        self.assertEquals(
            [2, 3],
            [month.get('number') for month in months_2013])

        self.assertEquals(
            [1],
            [month.get('number') for month in months_2012])



from ftw.contentpage.interfaces import ITeaser
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD
from plone.testing.z2 import Browser
from Products.CMFCore.utils import getToolByName
from pyquery import PyQuery
from StringIO import StringIO
from unittest2 import TestCase
import transaction


class TestTeaserImage(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestTeaserImage, self).setUp()
        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()
        self.catalog = getToolByName(self.portal, 'portal_catalog')

        self.contentpage = self.portal.get(
            self.portal.invokeFactory('ContentPage', 'contentpage'))
        # Fire all necessary events
        self.contentpage.processForm()

        transaction.commit()

        # Browser setup
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

    def _auth(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

    def test_teaser_interface(self):
        self.assertTrue(ITeaser.providedBy(self.contentpage),
            'ContentPage should provide ITeaser interface')

    def test_not_show_teaser(self):
        # No image, no description - no teaser
        self._auth()
        self.browser.open(self.contentpage.absolute_url())
        pq = PyQuery(self.browser.contents)
        self.assertFalse(
            pq('.simplelayout-content.sl-teaser-content-listing'),
            'There should be no simplelayout teaser viewlet')

    def test_show_teaser__description(self):
        self.contentpage.setDescription('qwerty')

        transaction.commit()

        self._auth()
        self.browser.open(self.contentpage.absolute_url())
        pq = PyQuery(self.browser.contents)

        wrapper = pq('.simplelayout-content.sl-teaser-content-listing')
        self.assertTrue(
            wrapper,
            'There should be the teaser viewlet')
        self.assertEquals(
            wrapper.find('.sl-text-wrapper')[0].text, 'qwerty')

        self.assertEquals(len(pq('.documentDescription')), 1,
            'The default description should appear only once')

    def test_show_teaser__image(self):
        self.contentpage.setImage(
            StringIO('GIF89a\x01\x00\x01\x00\x80\x00\x00\x00\x00\x00\x00\x00'
                '\x00!\xf9\x04\x04\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00'
                '\x01\x00\x00\x02\x02D\x01\x00;'))

        transaction.commit()

        self._auth()
        self.browser.open(self.contentpage.absolute_url())
        pq = PyQuery(self.browser.contents)

        wrapper = pq('.simplelayout-content.sl-teaser-content-listing')
        self.assertTrue(
            wrapper,
            'There should be the teaser viewlet')
        self.assertTrue(
            wrapper.find('.sl-img-wrapper'), 'No image found')

    def test_teaser_field_permission_contentpage(self):
        self._auth()
        self.browser.open("%s/edit" % self.contentpage.absolute_url())
        pq = PyQuery(self.browser.contents)
        self.assertTrue(pq('#fieldset-image .ArchetypesImageWidget'),
            'Expect an image field (teaser)')

        permissions = 'ftw.contentpage: Edit teaser image on ContentPage'
        self.contentpage.manage_permission(permissions, roles=[],
            acquire=False)
        self.contentpage.reindexObjectSecurity()
        transaction.commit()

        self.browser.open("%s/edit" % self.contentpage.absolute_url())
        pq = PyQuery(self.browser.contents)
        self.assertFalse(pq('#fieldset-image .ArchetypesImageWidget'),
            'Expect NO an image field (teaser)')

    def test_teaser_field_permission_eventpage(self):
        eventfolder = self.portal.get(self.portal.invokeFactory(
            'EventFolder', 'eventfolder'))
        eventfolder.processForm()

        eventpage = eventfolder.get(
            eventfolder.invokeFactory('EventPage', 'eventpage'))
        eventpage.processForm()
        transaction.commit()

        self._auth()
        self.browser.open("%s/edit" % eventpage.absolute_url())
        pq = PyQuery(self.browser.contents)
        self.assertTrue(pq('#fieldset-image .ArchetypesImageWidget'),
            'Expect an image field (teaser)')

        permissions = 'ftw.contentpage: Edit teaser image on EventPage'
        eventpage.manage_permission(permissions, roles=[],
            acquire=False)
        eventpage.reindexObjectSecurity()
        transaction.commit()

        self.browser.open("%s/edit" % eventpage.absolute_url())
        pq = PyQuery(self.browser.contents)
        self.assertFalse(pq('#fieldset-image .ArchetypesImageWidget'),
            'Expect NO an image field (teaser)')

    def test_teaser_field_permission_news(self):
        newsfolder = self.portal.get(self.portal.invokeFactory(
            'NewsFolder', 'newsfolder'))
        newsfolder.processForm()

        news = newsfolder.get(
            newsfolder.invokeFactory('News', 'news'))
        news.processForm()
        transaction.commit()

        self._auth()
        self.browser.open("%s/edit" % news.absolute_url())
        pq = PyQuery(self.browser.contents)
        self.assertTrue(pq('#fieldset-image .ArchetypesImageWidget'),
            'Expect an image field (teaser)')

        permissions = 'ftw.contentpage: Edit teaser image on News'
        news.manage_permission(permissions, roles=[],
            acquire=False)
        news.reindexObjectSecurity()
        transaction.commit()

        self.browser.open("%s/edit" % news.absolute_url())
        pq = PyQuery(self.browser.contents)
        self.assertFalse(pq('#fieldset-image .ArchetypesImageWidget'),
            'Expect NO an image field (teaser)')

    def test_teaser_viewlet_not_used_when_not_on_a_simplelayout_view(self):
        self.contentpage.setDescription('The Description')
        transaction.commit()
        self._auth()

        self.browser.open(self.contentpage.absolute_url())
        self.assertTrue(
            self.teaser_viewlet_visible(),
            'Expected the teaser viewlet to be visible on a normal content page.')

        self.browser.open('/'.join((self.contentpage.absolute_url(),
                                    'folder_contents')))
        self.assertFalse(
            self.teaser_viewlet_visible(),
            'Unexpectedly found traces of the teaser viewlet on folder_contents.')

    def teaser_viewlet_visible(self):
        pq = PyQuery(self.browser.contents)
        return pq('.sl-teaser-content-listing')



from ftw.contentpage.config import HAS_CONTENT_LISTING_BEHAVIOR
from ftw.contentpage.interfaces import ISummaryListingEnabled
from plone.app.layout.viewlets import ViewletBase
from plone.dexterity.interfaces import IDexterityContent
from plone.memoize import instance
from Products.Archetypes.interfaces import IBaseObject
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


if HAS_CONTENT_LISTING_BEHAVIOR:
    from ftw.contentpage.behaviors.content_categories import IContentCategories


class ContentListingViewlet(ViewletBase):
    """Lists content by categories"""
    def render(self):
        if ISummaryListingEnabled.providedBy(self.view):
            return ViewPageTemplateFile('content_summary_listing.pt')(self)
        return ViewPageTemplateFile('contentlisting.pt')(self)

    def available(self):
        return bool(self.get_content())

    @instance.memoize
    def get_content(self):
        query = {
            'sort_on': 'sortable_title',
            'object_provides': 'ftw.contentpage.interfaces.ICategorizable'}
        contents = self.context.getFolderContents(contentFilter=query,
                                                  full_objects=True)
        return self._create_resultmap(contents)

    def _create_resultmap(self, contents=None):
        resultmap = {}
        if not contents:
            return []
        for obj in contents:
            if IBaseObject.providedBy(obj):
                categories = obj.Schema()['content_categories'].get(obj)
            elif IDexterityContent.providedBy(obj) and HAS_CONTENT_LISTING_BEHAVIOR:
                categories = [item.encode('utf-8') for item in
                              IContentCategories(obj).content_categories]

            for cat in categories:
                if cat not in resultmap:
                    resultmap[cat] = []
                resultmap[cat].append((obj.title_or_id(),
                                       obj.absolute_url(),
                                       obj.Description()))

        items = resultmap.items()
        items.sort(key=lambda x: x[0].lower())
        return items



import transaction
from ftw.builder import Builder, create
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.registry.interfaces import IRegistry
from unittest2 import TestCase
from zope.component import getUtility


class TestListingBlockTableHelper(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        content_page = create(Builder('content page'))
        self.listing_block = create(Builder('listing block').within(content_page))
        self.file_ = create(Builder('file')
                            .with_dummy_content()
                            .having(description='S\xc3\xb6me description')
                            .within(self.listing_block))

    @browsing
    def test_file_link_default_value(self, browser):
        browser.login().visit(self.listing_block, view='block_view')
        self.assertEquals(
            self.file_.absolute_url() + '/download',
            browser.css('.linkWrapper a').first.attrib['href']
        )

    @browsing
    def test_file_link_empty_registry_value_without_trailing_slash(self, browser):
        registry = getUtility(IRegistry)
        registry['ftw.contentpage.listingblock.defaultfileviewname'] = u''
        transaction.commit()

        browser.login().visit(self.listing_block, view='block_view')
        self.assertEquals(
            self.file_.absolute_url(),
            browser.css('.linkWrapper a').first.attrib['href']
        )

    @browsing
    def test_file_link_changed_registry_value(self, browser):
        registry = getUtility(IRegistry)
        registry['ftw.contentpage.listingblock.defaultfileviewname'] = u'some-viewname'
        transaction.commit()

        browser.login().visit(self.listing_block, view='block_view')
        self.assertEquals(
            self.file_.absolute_url() + '/some-viewname',
            browser.css('.linkWrapper a').first.attrib['href']
        )



from simplelayout.base.views import SimpleLayoutView
from simplelayout.base.interfaces import ISimplelayoutView
from ftw.contentpage.interfaces import ISummaryListingEnabled
from zope.interface import implements


class SimpleLayoutSummaryView(SimpleLayoutView):

    implements(ISimplelayoutView, ISummaryListingEnabled)



from ftw.contentpage.testing import FTW_CONTENTPAGE_INTEGRATION_TESTING
from unittest2 import TestCase


class TestICSExport(TestCase):

    layer = FTW_CONTENTPAGE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.eventfolder = self.portal.get(self.portal.invokeFactory(
            'EventFolder', 'eventfolder'))
        self.eventfolder.processForm()

    def test_standard_export(self):
        kwargs = {'startDate': '2013-11-1 08:00',
                  'endDate': '2013-11-1 10:00',
                  'title': 'Event'}
        self.event = self.eventfolder.get(
            self.eventfolder.invokeFactory('EventPage', 'eventpage', **kwargs))

        address_args = {'address': 'Fakestreet 123',
                        'zip': '1234',
                        'city': 'mycity'}
        self.addressblock = self.event.get(
            self.event.invokeFactory('AddressBlock',
                                     'addresse',
                                     **address_args))

        view = self.event.restrictedTraverse('@@ics_view')
        view()
        result = view.feeddata()
        result_list = result.split('\n')

        expected = ['DTSTART:20131101T070000Z',
                    'DTEND:20131101T090000Z',
                    'LOCATION:Fakestreet 123\, 1234 mycity']
        self.assertTrue(set(expected).issubset(set(result_list)))

    def test_wholeday_export(self):
        kwargs = {'startDate': '2013-11-1',
                  'endDate': '2013-11-1',
                  'title': 'Event',
                  'wholeDay': True}
        self.event = self.eventfolder.get(
            self.eventfolder.invokeFactory('EventPage', 'eventpage', **kwargs))

        address_args = {'address': 'Fakestreet 123',
                        'zip': '1234',
                        'city': 'mycity'}
        self.addressblock = self.event.get(
            self.event.invokeFactory('AddressBlock',
                                     'addresse',
                                     **address_args))
        view = self.event.restrictedTraverse('@@ics_view')
        view()
        result = view.feeddata()
        result_list = result.split('\n')

        expected = ['DTSTART:20131031T230000Z',
                    'DTEND:20131031T230000Z',
                    'LOCATION:Fakestreet 123\, 1234 mycity']
        self.assertTrue(set(expected).issubset(set(result_list)))

    def test_multiday_export(self):
        kwargs = {'startDate': '2013-11-1',
                  'endDate': '2013-11-2',
                  'title': 'Event',
                  'wholeDay': True}
        self.event = self.eventfolder.get(
            self.eventfolder.invokeFactory('EventPage', 'eventpage', **kwargs))

        address_args = {'address': 'Fakestreet 123',
                        'zip': '1234',
                        'city': 'mycity'}
        self.addressblock = self.event.get(
            self.event.invokeFactory('AddressBlock',
                                     'addresse',
                                     **address_args))

        view = self.event.restrictedTraverse('@@ics_view')
        view()
        result = view.feeddata()
        result_list = result.split('\n')

        expected = ['DTSTART:20131031T230000Z',
                    'DTEND:20131101T230000Z',
                    'LOCATION:Fakestreet 123\, 1234 mycity']
        self.assertTrue(set(expected).issubset(set(result_list)))

    def test_export_without_addressblock(self):
        kwargs = {'startDate': '2013-11-1 08:00',
                  'endDate': '2013-11-1 10:00',
                  'title': 'Event',
                  'location': 'Towncentre'}
        self.event = self.eventfolder.get(
            self.eventfolder.invokeFactory('EventPage', 'eventpage', **kwargs))

        view = self.event.restrictedTraverse('@@ics_view')
        view()
        result = view.feeddata()
        result_list = result.split('\n')

        expected = ['LOCATION:Towncentre']
        self.assertTrue(set(expected).issubset(set(result_list)))



from ftw.upgrade import UpgradeStep
from Products.CMFCore.utils import getToolByName


class FixPortalTypeActionTitles(UpgradeStep):

    def __call__(self):

        ttool = getToolByName(self.portal, 'portal_types')

        fti_listingblock = ttool.get('ListingBlock')
        for action in fti_listingblock._actions:
            if action.id == 'sl-dummy-dummy-dummy':
                action.title = 'Listing'
            if action.id == 'sl-dummy-dummy-gallery':
                action.title = 'Gallery'

        fti_listingblock = ttool.get('AddressBlock')
        for action in fti_listingblock._actions:
            if action.id == 'sl-dummy-dummy-dummy':
                action.title = 'Map'



from Acquisition._Acquisition import aq_inner
from Acquisition._Acquisition import aq_parent
from ftw.contentpage import _
from ftw.contentpage.interfaces import INewsListingView
from ftw.contentpage.portlets.base_archive_portlet import ArchiveSummary
from plone.app.portlets.browser.interfaces import IPortletAddForm
from plone.app.portlets.browser.interfaces import IPortletEditForm
from plone.app.portlets.interfaces import IPortletPermissionChecker
from plone.app.portlets.portlets import base
from plone.memoize.view import memoize
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import form, button, field
from zope import schema
from zope.component import getMultiAdapter
from zope.i18n import translate
from zope.interface import implements


class INewsArchivePortlet(IPortletDataProvider):
    """Archive portlet interface.
    """
    portlet_title = schema.TextLine(
        title=_(u'news_archive_portlet_title_label', default=u'Title'),
        description=_(u'news_archive_portlet_title_description',
                      default=u'Use this to override the title of the portlet.'),
        required=False,
        default=u''
    )


class Assignment(base.Assignment):
    implements(INewsArchivePortlet)

    def __init__(self, portlet_title=''):
        self.portlet_title = portlet_title

    @property
    def title(self):
        return "News Archive Portlet"


class Renderer(base.Renderer):
    def __init__(self, context, request, view, manager, data):
        self.context = context
        self.data = data
        self.request = request
        self.view = view

    @property
    def available(self):
        """Only show the portlet:
        - If there are News
        - If view is NewsListing
        """
        has_news = bool(self.archive_summary())

        if INewsListingView.providedBy(self.view):
            return has_news
        else:
            return False

    @memoize
    def archive_summary(self):
        """Returns an ordered list of summary infos per month."""
        return ArchiveSummary(
            self.context,
            self.request,
            ['ftw.contentpage.interfaces.INews'],
            'effective',
            'newslisting')()

    def get_portlet_title(self):
        default_title = translate(_(u'Archive'), context=self.request)
        return getattr(self.data, 'portlet_title', '') or default_title

    render = ViewPageTemplateFile('news_archive_portlet.pt')


class AddForm(form.AddForm):
    implements(IPortletAddForm)
    label = _(u'news_archive_portlet_add_form_label',
              default=u'Add News Archive Portlet')
    description = _(u'news_archive_portlet_add_form_description',
                    default=u'Renders a widget containing the number of news by year and month.')

    fields = field.Fields(INewsArchivePortlet)

    def __init__(self, context, request):
        super(AddForm, self).__init__(context, request)
        self.status = None
        self._finishedAdd = None

    def __call__(self):
        IPortletPermissionChecker(aq_parent(aq_inner(self.context)))()
        return super(AddForm, self).__call__()

    def nextURL(self):
        editview = aq_parent(aq_inner(self.context))
        context = aq_parent(aq_inner(editview))
        url = str(getMultiAdapter((context, self.request),
                                  name=u"absolute_url"))
        return url + '/@@manage-portlets'

    @button.buttonAndHandler(_(u"label_save", default=u"Save"), name='add')
    def handleAdd(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        obj = self.createAndAdd(data)
        if obj is not None:
            # mark only as finished if we get the new object
            self._finishedAdd = True

    @button.buttonAndHandler(_(u"label_cancel", default=u"Cancel"),
                             name='cancel_add')
    def handleCancel(self, action):
        nextURL = self.nextURL()
        return self.request.response.redirect(nextURL)

    def add(self, object_):
        ob = self.context.add(object_)
        self._finishedAdd = True
        return ob

    def create(self, data):
        return Assignment(
            portlet_title=data.get('portlet_title', ''),
        )


class EditForm(form.EditForm):
    implements(IPortletEditForm)
    label = _(u'news_archive_portlet_edit_form_label',
              default=u'Edit News Archive Portlet')
    description = _(u'newst_archive_portlet_edit_form_description',
                    default=u'Renders a widget containing the number of news by year and month.')

    fields = field.Fields(INewsArchivePortlet)

    def __init__(self, context, request):
        super(EditForm, self).__init__(context, request)
        self.status = None
        self._finishedAdd = None

    def __call__(self):
        IPortletPermissionChecker(aq_parent(aq_inner(self.context)))()
        return super(EditForm, self).__call__()

    def nextURL(self):
        editview = aq_parent(aq_inner(self.context))
        context = aq_parent(aq_inner(editview))
        url = str(getMultiAdapter((context, self.request),
                                  name=u"absolute_url"))
        return url + '/@@manage-portlets'

    @button.buttonAndHandler(_(u"label_save", default=u"Save"), name='apply')
    def handleSave(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        changes = self.applyChanges(data)
        if changes:
            self.status = "Changes saved"
        else:
            self.status = "No changes"

        nextURL = self.nextURL()
        return self.request.response.redirect(nextURL)

    @button.buttonAndHandler(_(u"label_cancel", default=u"Cancel"),
                             name='cancel_edit')
    def handleCancel(self, action):
        nextURL = self.nextURL()
        return self.request.response.redirect(nextURL)



from ftw.builder import Builder
from ftw.builder import create
from ftw.contentpage.browser import authorities
from ftw.contentpage.interfaces import IAuthority
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from ftw.contentpage.tests.pages import AuthoritiesView
from ftw.testing import MockTestCase
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest2 import TestCase
import transaction


class TestHelperFunctions(MockTestCase):

    def setUp(self):
        super(TestHelperFunctions, self).setUp()

        self.brain_mocks = [
            self.create_dummy(Title='Foo',
                              getPath=lambda: '/plone/foo',
                              getURL=lambda: 'http://nohost/plone/foo'),

            self.create_dummy(Title='Bar',
                              getPath=lambda: '/plone/foo/bar',
                              getURL=lambda: 'http://nohost/plone/foo/bar'),

            self.create_dummy(Title='Baz',
                              getPath=lambda: '/plone/foo/baz',
                              getURL=lambda: 'http://nohost/plone/foo/baz')]

    def test_get_brain_data(self):
        self.assertEqual(map(authorities.get_brain_data, self.brain_mocks),

                         [{'title': 'Foo',
                           'path': '/plone/foo',
                           'url': 'http://nohost/plone/foo',
                           'children': []},

                          {'title': 'Bar',
                           'path': '/plone/foo/bar',
                           'url': 'http://nohost/plone/foo/bar',
                           'children': []},

                          {'title': 'Baz',
                           'path': '/plone/foo/baz',
                           'url': 'http://nohost/plone/foo/baz',
                           'children': []}])

    def test_make_treeish(self):
        data = map(authorities.get_brain_data, self.brain_mocks)

        self.maxDiff = None
        self.assertEqual(
            authorities.make_treeish(data),

            [{'title': 'Foo',
              'path': '/plone/foo',
              'url': 'http://nohost/plone/foo',

              'children': [

                        {'title': 'Bar',
                         'path': '/plone/foo/bar',
                         'url': 'http://nohost/plone/foo/bar',
                         'children': []},

                        {'title': 'Baz',
                         'path': '/plone/foo/baz',
                         'url': 'http://nohost/plone/foo/baz',
                         'children': []},

                        ]},
             ])


class TestTreeView(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_only_first_two_levels_visible(self):
        container = create(Builder('content page'))

        first = create(Builder('content page')
                       .titled('First')
                       .providing(IAuthority)
                       .within(container))

        second = create(Builder('content page')
                        .titled('Second')
                        .providing(IAuthority)
                        .within(first))

        create(Builder('content page')
               .titled('Third')
               .providing(IAuthority)
               .within(second))

        AuthoritiesView().visit_on(container)
        self.assertEquals(['First', 'Second'], AuthoritiesView().link_labels)

    def test_view_has_two_columns(self):
        container = create(Builder('content page'))

        create(Builder('content page')
               .titled('One')
               .providing(IAuthority)
               .within(container))

        create(Builder('content page')
               .titled('Two')
               .providing(IAuthority)
               .within(container))

        AuthoritiesView().visit_on(container)
        self.assertEquals([['One'], ['Two']],
                          AuthoritiesView().link_labels_per_column)

    def test_only_pages_with_orgunit_marker_interface_are_displayed(self):
        container = create(Builder('content page'))

        create(Builder('content page')
               .titled('With IAuthority')
               .providing(IAuthority)
               .within(container))

        create(Builder('content page')
               .titled('Without IAuthority')
               .within(container))

        AuthoritiesView().visit_on(container)
        self.assertEquals(['With IAuthority'],
                          AuthoritiesView().link_labels)

    def test_pages_are_linked_properly(self):
        container = create(Builder('content page'))

        foo = create(Builder('content page')
                     .titled('Foo')
                     .providing(IAuthority)
                     .within(container))

        bar = create(Builder('content page')
                     .titled('Bar')
                     .providing(IAuthority)
                     .within(container))

        AuthoritiesView().visit_on(container)
        self.assertEquals(
            [foo.absolute_url(),
             bar.absolute_url()],

            [AuthoritiesView().link_url('Foo'),
             AuthoritiesView().link_url('Bar')])

    def test_column_balancing_can_be_changed_with_property(self):
        container = create(Builder('content page'))

        create(Builder('content page')
               .titled('One')
               .providing(IAuthority)
               .within(container))

        create(Builder('content page')
               .titled('Two')
               .providing(IAuthority)
               .within(container))

        create(Builder('content page')
               .titled('Three')
               .providing(IAuthority)
               .within(container))

        create(Builder('content page')
               .titled('Four')
               .providing(IAuthority)
               .within(container))

        AuthoritiesView().visit_on(container)
        self.assertEquals([['One', 'Two'],
                           ['Three', 'Four']],
                          AuthoritiesView().link_labels_per_column)

        container._setProperty('leftcolumn_num_elements', 1, 'int')
        transaction.commit()
        AuthoritiesView().visit_on(container)
        self.assertEquals([['One'],
                           ['Two', 'Three', 'Four']],
                          AuthoritiesView().link_labels_per_column)

    def test_pages_excluded_from_navigation_are_not_shown(self):
        container = create(Builder('content page'))

        create(Builder('content page')
               .titled('Shown In Navigation')
               .providing(IAuthority)
               .within(container))

        create(Builder('content page')
               .titled('Excluded From Navigation')
               .providing(IAuthority)
               .within(container)
               .having(excludeFromNav=True))

        AuthoritiesView().visit_on(container)
        self.assertEquals(['Shown In Navigation'],
                          AuthoritiesView().link_labels)



from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.testing.z2 import Browser
from unittest2 import TestCase
import transaction


class TestNewsRssListing(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.newsfolder = create(Builder('news folder')
                                 .titled('Ein N\xc3\xbcwsFolder'))
        self.news = create(Builder('news').within(self.newsfolder))

        transaction.commit()

    def login(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD,))

    def test_newslisting_rss_items(self):
        self.browser.open(self.newsfolder.absolute_url() + '/news_rss_listing')

        rdf = '<rdf:li rdf:resource="{0}"/>'.format(
            self.news.absolute_url())
        self.assertIn(rdf,
                      self.browser.contents,
                      'Did not found the rdf tag for the news')

        link = '<link>{0}</link>'.format(self.news.absolute_url())
        self.assertIn(link,
                      self.browser.contents,
                      'Did not found the link tag for the news')

    @browsing
    def test_newsitem_contains_pubdate(self, browser):
        browser.open(self.newsfolder, view='news_rss_listing')
        browser.parse_as_html()  # use HTML parser so that we have no XML namespaces.

        effective_date = self.news.getEffectiveDate()
        self.assertEqual(
            # Difference between "%e" and "%-e":
            # %e has a leading space on single numbers - thats why the tests
            # failing between the 1st and the 9th every month :-)
            # %-e Removes the leading space - only works on unix machines.
            effective_date.strftime('%a, %-e %b %Y %H:%M:%S %z').strip(),
            browser.css('rdf item pubDate').first.text
        )



from DateTime import DateTime
from DateTime.interfaces import SyntaxError as dtSytaxError
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.PloneBatch import Batch
from zope.publisher.browser import BrowserView
from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.permissions import AccessInactivePortalContent



def extend_query_by_date(query, datestring, date_field):
    try:
        start = DateTime(datestring)
    except dtSytaxError:
        return query
    end = DateTime('%s/%s/%s' % (start.year() + start.month() / 12,
                                 start.month() % 12 + 1, 1))
    end = end - 1
    query[date_field] = {'query': (start.earliestTime(),
                                    end.latestTime()),
                          'range': 'minmax'}
    return query


class BaseListing(BrowserView):

    def __init__(self, context, request):
        super(BaseListing, self).__init__(context, request)
        self.batch = None

    def __call__(self):
        b_start = self.request.form.get('b_start', 0)
        self.batch = Batch(self.get_items(), 10,
                           b_start)
        return self.template()

    def get_creator(self, item):
        memberid = item.Creator
        mt = getToolByName(self.context, 'portal_membership')
        member_info = mt.getMemberInfo(memberid)
        if member_info:
            fullname = member_info.get('fullname', '')
        else:
            fullname = None
        if fullname:
            return fullname
        else:
            return memberid

    def search_results(self, query, date_field):
        """Get all news items"""

        # Implement archive functionality - used by the archive portlet
        context = self.context
        ct = context.portal_type
        datestring = self.request.get('archive')
        if datestring:
            query = extend_query_by_date(query, datestring, date_field)

        if ct == 'Topic':
            return context.queryCatalog()
        else:
            show_inactive = _checkPermission(AccessInactivePortalContent,
                                             context)
            catalog = getToolByName(context, 'portal_catalog')
            query['path'] = '/'.join(context.getPhysicalPath())
            return catalog(query, show_inactive=show_inactive)

    def has_img(self, brain):
        """ Checks if the news have an image.
        """
        return bool(brain.getObject().getImage())

    def get_img(self, brain, width=100, height=100):
        obj = brain.getObject()
        scale = obj.restrictedTraverse('@@images')
        return scale.scale(
            'image',
            width=width,
            height=height).tag(**{'class': 'tileImage'})



from AccessControl import ClassSecurityInfo
from archetypes.referencebrowserwidget import ReferenceBrowserWidget
from ftw.contentpage import _
from ftw.contentpage import config
from ftw.contentpage.content.schema import finalize
from ftw.contentpage.interfaces import ITextBlock
from plone.app.blob.field import ImageField
from Products.Archetypes.public import DisplayList
from Products.Archetypes.public import SelectionWidget
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.lib.historyaware import HistoryAwareMixin
from Products.CMFCore.utils import getToolByName
from simplelayout.base.interfaces import ISimpleLayoutBlock
from zope.interface import implements

from Products.ATContentTypes.config import HAS_LINGUA_PLONE
if HAS_LINGUA_PLONE:
    from Products.LinguaPlone import public as atapi
else:
    from Products.Archetypes import atapi


default_schema = atapi.Schema((
    atapi.BooleanField(
        name='showTitle',
        schemata='default',
        default=0,
        widget=atapi.BooleanWidget(
            label=_(u'label_show_title',
                    default=u'Show Title'))),

    atapi.TextField(
        name='text',
        primary=True,
        required=False,
        searchable=True,
        allowable_content_types=('text/html', ),
        default_content_type='text/html',
        validators=('isTidyHtmlWithCleanup', ),
        default_input_type='text/html',
        default_output_type='text/x-html-safe',
        widget=atapi.RichWidget(
            label=_(u'label_text',
                    default=u'Text'),
            allow_file_upload=False,
            rows=25)),

))

image_schema = atapi.Schema((
    ImageField(
        name='image',
        required=False,
        original_size=config.ORIGINAL_SIZE,
        schemata='image',
        widget=atapi.ImageWidget(
            label=_(u'label_image',
                    default=u'Image'))),

    atapi.BooleanField(
        name='imageClickable',
        schemata='image',
        default=0,
        widget=atapi.BooleanWidget(
            label=_(u'label_image_clickable',
                    default=u'Image clickable'),
            description=_(u'description_image_clickable',
                          default=u'Opens image in an overlay'))),

    atapi.StringField(
        name='imageCaption',
        required=False,
        searchable=True,
        schemata='image',
        widget=atapi.StringWidget(
            label=_(u'label_image_caption',
                    default=u'Image'))),

    atapi.StringField(
        name='imageAltText',
        schemata='image',
        required=False,
        widget=atapi.StringWidget(
            label=_(u'label_image_alt_text',
                    default=u'Image'),
            description=_(u'description_image_alt_text',
                          default=u'Enter an alternative text '
                                  u'for the image'))),
))

teaser_schema = atapi.Schema((

    atapi.StringField(
        name="teaserSelectLink",
        schemata='teaser',
        multiValued=False,
        storage=atapi.AttributeStorage(),
        vocabulary=DisplayList((
            ('intern', _(u'teaser_internal_reference',
                         default="Internal Reference")),
            ('extern', _(u'teaser_external_url',
                         default="External URL")),
        )),
        widget=SelectionWidget(
            label=_(u'teaser_select_label',
                    default="Select internal or external link"),
            )
    ),

    atapi.StringField(
        name='teaserExternalUrl',
        schemata='teaser',
        searchable=False,
        validators='isURL',
        widget=atapi.StringWidget(
            label=_(u'teaser_external_url',
                    default=u'External URL'))),

    atapi.ReferenceField(
        name='teaserReference',
        relationship='teasesContent',
        schemata='teaser',
        multiValued=False,
        widget=ReferenceBrowserWidget(
            force_close_on_insert=True,
            label=_(
                u'teaser_internal_reference',
                default=u'Internal Reference'
            ),
        )),
))

textblock_schema = ATContentTypeSchema.copy() + \
    default_schema.copy() + image_schema.copy() + teaser_schema.copy()

textblock_schema['title'].required = False
textblock_schema['text'].widget.filter_buttons = ('image', )

finalize(textblock_schema, hide=['description'])

# Protect the teaser schema with a specific permission
permission = "ftw.contentpage: Add teaser link"
for name in teaser_schema.keys():
    textblock_schema[name].write_permission = permission


class TextBlock(ATCTContent, HistoryAwareMixin):
    """Textblock for Contentpage
    """
    security = ClassSecurityInfo()
    implements(ITextBlock, ISimpleLayoutBlock)
    schema = textblock_schema

    # Special workarround for empty titles, otherwise we have "[...]"
    # results in the search function
    def setTitle(self, value):
        portal_transforms = getToolByName(self, 'portal_transforms')
        field = self.schema['title']
        if not value:
            new_value = self.REQUEST.get('text', None)
            if new_value is not None:
                converted = portal_transforms.convertTo(
                    'text/plain',
                    new_value).getData().replace('\r', '').replace('\n', '')
                crop = self.restrictedTraverse('@@plone').cropText
                cropped = crop(converted, 30, '')
                field.set(self, cropped.strip())
        else:
            field.set(self, value)

atapi.registerType(TextBlock, config.PROJECTNAME)



import unittest2 as unittest
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
import os
import transaction
from plone.testing.z2 import Browser
from zope.viewlet.interfaces import IViewletManager
from Products.Five.browser import BrowserView
from zope.component import queryMultiAdapter
from Products.CMFCore.utils import getToolByName


class TestByline(unittest.TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING


    def setUp(self):
        self.portal = self.layer['portal']

        self.newsfolder = self.portal.get(self.portal.invokeFactory(
                'NewsFolder', 'newsfolder'))
        image_ = open("%s/dummy.png" % os.path.split(__file__)[0])

        self.news = self.newsfolder.get(self.newsfolder.invokeFactory(
                'News', 'news', title="My News", image=image_))
        transaction.commit()
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False


    def _get_viewlet(self):
        view = BrowserView(self.news, self.news.REQUEST)
        manager_name = 'plone.belowcontenttitle'
        manager = queryMultiAdapter(
            (self.news, self.news.REQUEST, view),
            IViewletManager,
            manager_name)
        self.failUnless(manager)
        # Set up viewlets
        manager.update()
        name = 'plone.belowcontenttitle.documentbyline'
        return [v for v in manager.viewlets if v.__name__ == name]

    def test_byline_noworkflow(self):
        viewlet = self._get_viewlet()
        self.assertEqual(viewlet[0].getWorkflowState(), None)

    def test_byline_wf(self):
        self.wf_tool = getToolByName(self.portal, 'portal_workflow')
        self.wf_tool.setDefaultChain('simple_publication_workflow')
        viewlet = self._get_viewlet()
        self.assertEqual(viewlet[0].getWorkflowState(), 'Private')

    def test_byline_onestateworkflow(self):
        viewlet = self._get_viewlet()
        self.wf_tool = getToolByName(self.portal, 'portal_workflow')
        self.wf_tool.setDefaultChain('one_state_workflow')
        self.assertEqual(viewlet[0].getWorkflowState(), 'Published')
